/**
 * Using EnOcean Pushbutton Transmitter (PTM210) with EnOcean Pi
 * The following code uses Raspberry Pi pin 11, 12, 13 and 15 to
 * control 4 external devices (e.g. LED/Relay)
 *
 * a short click turns on the device
 * a long click (relase/push interval > 1s) turns off the device
 *
 * http://www.kerrywong.com
 *
 * 8/2014
 */
#define SER_PORT "/dev/ttyAMA0"
#define ROCKER_DEVICE_ID 0x294993
#include "./eoLink.h"
#include <stdio.h>
#include <time.h>
#include <wiringPi.h>

const int BTN_A = 0;
const int BTN_B = 1;
const int BOW_DIR = 2;

uint8_t rdata[4];
uint8_t rdataPrev[4];

eoGateway gateway;
eoProfile *profile;

uint16_t recv;
bool rockerAPressed = false;
bool rockerBPressed = false;
struct timespec tStart, tEnd;

void init()
{
    gateway.Open(SER_PORT);
    gateway.TeachInModule->SetRPS(0x02, 0x01);
    gateway.LearnMode=true;

    wiringPiSetup();

    for (int i = 0; i < 4; i++)
    {
        pinMode(i, OUTPUT);
        digitalWrite(i, LOW);
    }
}

void checkStatus()
{
    for (int i = 0; i<profile->GetChannelCount(); i++)
    {
        rdataPrev[i] = rdata[i];

        if (profile->GetValue( profile->GetChannel(i)->type, rdata[i]) == EO_OK) 
        {
            eoChannelInfo *info = profile->GetChannel(i);
            printf("--%s::%u \n", info->ToString(NAME),rdata[i]);
        }
    }

    if (rdata[BTN_A] < 2 && rdata[BTN_B] == 2 && rdata[BOW_DIR] == 1) // one of the buttons on rocker A is depressed
    {
        rockerAPressed = true;
        rockerBPressed = false;
        clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &tStart);
        printf("ROCKER A\n");
    } 
    else if (rdata[BTN_B] < 2 && rdata[BTN_A] == 2 && rdata[BOW_DIR] == 1) // one of the buttons on rocker B is depressed
    {
        rockerBPressed = true;
        rockerAPressed = false;
        clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &tStart);
        printf("ROCKER B\n");
    } 
    else if (rdata[BOW_DIR] == 0) // rocker released
    {
        clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &tEnd);
        double t = (double) (tEnd.tv_sec - tStart.tv_sec) * 1.0e9 + (double)(tEnd.tv_nsec - tStart.tv_nsec);
        t = t/1.0e6; //ms

        if (t < 1000.0d) //short click turns on
        {
            if (rockerAPressed && rdataPrev[BTN_A] == 0) digitalWrite(0, HIGH);
            if (rockerAPressed && rdataPrev[BTN_A] == 1) digitalWrite(1, HIGH);
            if (rockerBPressed && rdataPrev[BTN_B] == 0) digitalWrite(2, HIGH);
            if (rockerBPressed && rdataPrev[BTN_B] == 1) digitalWrite(3, HIGH);
        } 
        else //long click turns off
        {
            if (rockerAPressed && rdataPrev[BTN_A] == 0) digitalWrite(0, LOW);
            if (rockerAPressed && rdataPrev[BTN_A] == 1) digitalWrite(1, LOW);
            if (rockerBPressed && rdataPrev[BTN_B] == 0) digitalWrite(2, LOW);
            if (rockerBPressed && rdataPrev[BTN_B] == 1) digitalWrite(3, LOW);
        }

        rockerAPressed = false;
        rockerBPressed = false;
    }
}

int main()
{
    init();

    while(1) 
    {
        recv = gateway.Receive();

        if (recv & RECV_PROFILE) 
        {
            profile = gateway.device->GetProfile();
            if (gateway.device->ID != ROCKER_DEVICE_ID) break;

            checkStatus();
        }
    }

    return 0;
}
